angular.module("textViewer").factory("eyeTrackerFactory", function () {

    var rightX;
    var rightY;
    var leftX;
    var leftY;
    var  socket;

    var lastLeftX = [];
    var lastLeftY = [];
    var lastRightX = [];
    var lastRightY = [];

    var cursorMeanBufferSize = 30;
    var offsetX= 0;
    var offsetY= 0; 

    var finalPosX=0;
    var finalPosY=0;

    var init = function(){
	initSocket();
    }
    
    var update = function(){
	socket.emit('updateRequest', null);
    }

    var getPos = function(){
	return [finalPosX, finalPosY];
    }

    function initSocket(){
	socket = io("http://localhost:3000");
	//When data received from the server
	socket.on('coordinates', function( leftX, leftY, rightX, rightY ){
	    
	    updateArrays(leftX, leftY, rightX, rightY);
	    updateCursor();
	    update();
	    

	});
	
	window.onmousemove = function(e){
	    offsetX = -1 * (e.clientX-e.screenX);
	    offsetY = -1 * (e.clientY-e.screenY);
	};
    }


    //Averaging the last "cursorMeanBufferSize" values received    
    function updateCursor(){

	var averageLeftX = 0;
	var averageLeftY = 0;
	var averageRightX = 0;
	var averageRightY = 0;
	
	for(i=0 ; i < lastLeftX.length ; i++)
	    averageLeftX += lastLeftX[i];

	for(i=0 ; i < lastLeftY.length ; i++)
	    averageLeftY += lastLeftY[i];

	averageLeftX = averageLeftX/lastLeftY.length;
	averageLeftY = averageLeftY/lastLeftY.length;

	var leftX = averageLeftX*screen.width-offsetX;
	var leftY = averageLeftY*screen.height-offsetY; 

	for(i=0 ; i < lastRightX.length ; i++)
	    averageRightX += lastRightX[i];

	for(i=0 ; i < lastRightY.length ; i++)
	    averageRightY += lastRightY[i];

	averageRightX = averageRightX/lastRightX.length;
	averageRightY = averageRightY/lastRightX.length;

	var rightX = averageRightX*screen.width-offsetX;
	var rightY = averageRightY*screen.height-offsetY;
	
	finalPosX = (rightX+leftX)/2;
	finalPosY = (rightY+leftY)/2;
	
    };

    //Kepping only the last "cursorMeanBufferSize" values 
    function updateArrays(leftX, leftY, rightX, rightY){

	//Filtering eyes blinking
	if( (leftX == 0. || leftY == 0.) && (rightX == 0. || rightY == 0.) )
	    return;

	if( leftX == 0. || leftY == 0. ){
	    leftX = rightX;
	    leftY = rightY;
	}
	
	if( rightX == 0. || rightY == 0. ){
	    rightX = leftX;
	    rightY = leftY;
	}
	
	
	lastLeftX.unshift(leftX);
	lastLeftY.unshift(leftY);

	if( lastLeftX.length > cursorMeanBufferSize ){
	    tmp = [];
	    
	    for( i = 0 ; i < cursorMeanBufferSize ; i++ )
		tmp.push(lastLeftX[i]);

	    lastLeftX = tmp;
	}

	if( lastLeftY.length > cursorMeanBufferSize ){
	    tmp = [];
	    
	    for( i=0 ; i < cursorMeanBufferSize ; i++ )
		tmp.push(lastLeftY[i]);

	    lastLeftY = tmp;
	}

	lastRightX.unshift(rightX);
	lastRightY.unshift(rightY);

	if( lastRightX.length > cursorMeanBufferSize ){
	    tmp = [];
	    
	    for( i = 0 ; i < cursorMeanBufferSize ; i++ )
		tmp.push(lastRightX[i]);

	    lastRightX = tmp;
	}

	if( lastRightY.length > cursorMeanBufferSize ){
	    tmp = [];
	    
	    for( i = 0 ; i < cursorMeanBufferSize ; i++ )
		tmp.push(lastRightY[i]);

	    lastRightY = tmp;
	}
    }
    
    return {
	update : update,
	init : init,
	getPos : getPos
    };
    
});
