angular.module("textViewer").controller('textViewerController', ['$scope', 'eyeTrackerFactory', '$interval', 'TextModel', function($scope, eyeTrackerFactory, $interval, TextModel){

    $scope.text = new TextModel();

    eyeTrackerFactory.init();
    
    $scope.updatePos = function(){
	eyeTrackerFactory.update();
    }

    $scope.updateDisplay = function(){
	
	$scope.pos = eyeTrackerFactory.getPos();
	$scope.displayPos = eyeTrackerFactory.getPos();
	$scope.displayPos[0]+=window.pageXOffset;
	$scope.displayPos[1]+=window.pageYOffset;
	
	if($scope.viewRangeMode)
	    updateViewRangeMode();
	if($scope.readMode)
	    updateReadMode();
    }

    $scope.updatePos();
    $interval($scope.updatePos, 10000);
    $interval($scope.updateDisplay, 1);
    

    /* debugging code, uncomment to use mouse as eye tracker*/
    /*
       window.onmousemove = function(e)
       {
       //console.log(e.clientX+" "+ e.ClientY+" "+ e.screenX+" "+ e.screenY+" "+ e.clientX-e.screenX+" "+ e.clientY-e.screenY);
       $scope.$apply(function(){
       $scope.pos = [e.clientX, e.clientY];
       $scope.displayPos = [e.clientX+window.pageXOffset, e.clientY+window.pageYOffset];
       if($scope.viewRangeMode)
       updateViewRangeMode();
       if($scope.readMode)
       updateReadMode();
       });
       }
       //*/
    
    $scope.fontSize = 25;
    $scope.lineSpace =14;
    $scope.readModeTime = 65; //Minimal time (in milliseconds) needed to consider that a word has been read
    
    $scope.showRect = false;
    $scope.showReticle = false;
    $scope.showParams = false;
    $scope.readMode = true;
    $scope.colorWords = false;
    $scope.doneReading = false;

    var thresholdParagraphRead = 0.5; // Threshold of the number of words marked in the paragraph at which we consder the paragraph has been read (here, 50%)

    $scope.nbWordsRead = function(){
	return $scope.text.getTotalMarkedWordsCount();
    }
    
    $scope.printModel = function (){
	console.log("textViewerController::Print Model() " + $scope.text.toString());
    } 


    /* Parses raw text and create the model */
    $scope.createModelFromText = function() {
	var tmp_elem = $scope.text.element;
	$scope.text = new TextModel();
	$scope.text.element = tmp_elem;
	$scope.text.createFromString($scope.text_brut);
    }

    $scope.init = function() {
	$scope.createModelFromText();
    }
    
    function addDisplayBoundingRect(r) {
	if(r == null)
	    console.log("null rect");
	else {
	    
	    x_pos = r.left+window.pageXOffset;
	    y_pos = r.top+window.pageYOffset;

	    div = document.createElement("div");
	    div.className = "boundingRectDisplay";
	    div.setAttribute("style","position:absolute;left:"+x_pos+"px;top:"+y_pos+"px;width:"+r.width+"px;height:"+r.height+"px;border:1px solid #000;pointer-events:none;")
	    document.body.appendChild(div);
	    
	}
    }

    $scope.afficherRects = function (){

	supprimerAffichageRects();
	
	if($scope.showRect){
	    return;
	}

	//display text bounding rect
	addDisplayBoundingRect($scope.text.element.getBoundingClientRect());
	
	for(i=0; i<$scope.text.paragraphs.length; ++i){
	    addDisplayBoundingRect($scope.text.paragraphs[i].element.getBoundingClientRect()); 
	    for(j=0; j<$scope.text.paragraphs[i].words.length; ++j)
		addDisplayBoundingRect($scope.text.paragraphs[i].words[j].element.getBoundingClientRect());	

	}
	
    }


    function supprimerAffichageRects(){
	
	brects = document.getElementsByClassName("boundingRectDisplay");
	
	r = [];
	
	for(i=0; i< brects.length; i++)
	    r[i] = brects[i];

	for(i=0; i<r.length; i++)
	    r[i].parentNode.removeChild(r[i]);
	
    }

    function processParagraph(p) {
	
	var n = p.getMarkedWordsCount();
	var nt = p.words.length;
	var ratio = n/nt;
	
	if(n/nt  > thresholdParagraphRead){
	    
	    p.read = true; 
	    var i; 
	    for(i=0; i<p.words.length; ++i){
		p.words[i].read= true;
		p.words[i].color = "rgb(0,0,255)";	
	    }
	    
	}
    }

    
    var word = null;
    var paragraph = null;
    var time = 0;
    function updateReadMode(){
	
	parse = true;
	if(word != null){
	    if( word.isInRect($scope.pos[0], $scope.pos[1]))
		parse = false; 
	    else{
		
		d = new Date();
		readTime = d.getTime() - time;
		
		if(readTime > $scope.readModeTime){
		    if(!word.marked){
			if($scope.colorWords)
			    word.color = "rgb(255,0,0)";
			
			word.marked = true;
		    }
		}
		
		word = null;
		time = 0;
	    }
	}
	if(parse){
	    if($scope.text.isInRect($scope.pos[0],$scope.pos[1])){
		
		console.log("in text");
		var p = null;
		p = $scope.text.getParagraphAt($scope.pos[0],$scope.pos[1]);
		if(p != null){
		    console.log("paragraph found");
		    if(p != paragraph && paragraph != null){
			console.log("new paragraph");
			processParagraph(paragraph);
			paragraph = null; 
		    }

		    var w = p.getWordAt($scope.pos[0], $scope.pos[1]);
		    if(w != null){
			d = new Date();
			time = d.getTime();
			word = w;
			paragraph = p;
			
		    }
		}
	    }
	} 
    }

}]);
