angular.module("textViewer").factory("eyeTrackerFactory", ['$interval', function ($interval) {

    var mouse_debug = false;//set to true to bypass the websocket connexion to the eye tracker to debug with mouse cursor
    var VARIANCE_THRESHOLD = 50; //threshold for the sum of var1 and var2 to determine if var1+var2 > VARIANCE_THRESHOLD then we have a saccade 
    var FIXATION_MIN_DURATION = 100;//minimum duration of a fixation in milliseconds
    
    var  socket;

    var mean_x = 0;
    var mean_y = 0;
    
    var var1 = 0;
    var var2 = 0;
    
    var angle = 0;
    
    var clearBufferTime;
    
    var bufferX = [];
    var bufferY = [];

    var X_it = 0;
    var Y_it = 0;
    
    const CURSOR_MEAN_BUFFER_SIZE = 50;
    
    var offsetX= 0;
    var offsetY= 0; 

    var finalPosX=0;
    var finalPosY=0;

    var fixationCallback = null;
    
    function setFixationCallback(f){
	fixationCallback = f;
    }

    var init = function(){
	if(mouse_debug == true)
	    $interval(generateNewRandomCoordinates, 10);
	else
	    initSocket();

	window.onmousemove = function(e){
	    
	    offsetX = -1 * (e.clientX-e.screenX);
	    offsetY = -1 * (e.clientY-e.screenY);
	    mouse_x = e.clientX;
	    mouse_y = e.clientY;
	};

    }

    //Send an update request to the eye tracker module
    var update = function(){
	if(!mouse_debug)
	    socket.emit('updateRequest', null);
    }
    

    var getPos = function(){
	
	return {
	    x : mean_x,
	    y : mean_y,
	    var_x : var1,
	    var_y : var2
	};
	
    }

    var mouse_x = -1;
    var mouse_y = -1;
    
    //debug
    function generateNewRandomCoordinates(){
	
	if(mouse_x >= 0 && mouse_y >= 0){
	    
	    var sig = 40;
	    
	    var x = mouse_x - sig/2 + Math.random() * (sig);
	    var y = mouse_y - sig/2 + Math.random() * (sig);
	    
	    addNewCoordinates(x, y, x, y);
	    
	    //console.log("rand");
	    // mean_x = mouse_x;
	    // mean_y = mouse_y;
	    // var1 = sig;
	    // var2 = sig;
	    // angle = 0;
	    
	}	
    }

    function addNewCoordinates(leftX, leftY, rightX, rightY){
	
	var old_mean_x = mean_x;
	var old_mean_y = mean_y;
	
	var old_var1 = var1;
	var old_var2 = var2;
	
	updateArrays(leftX, leftY, rightX, rightY);
	updateCursor();

	if(var1+var2 > VARIANCE_THRESHOLD){
	    var d = new Date();
	    var t = d.getTime();
	    
	    if(t-clearBufferTime>FIXATION_MIN_DURATION){
		if(fixationCallback != null)
		    fixationCallback({x : old_mean_x,
				      y : old_mean_y,
				      var_x : old_var1,
				      var_y : old_var2});
		
	    }
	    
	    clearBufferTime = t;
	    
	    bufferX.length = 0;
	    bufferY.length = 0;
	    
	    X_it = 0; 
	    Y_it = 0;
	    
	    updateArrays(leftX,leftY,rightX,rightY);
	    
	}
    }

    // Detects eyes blinking
    // Returns :
    // 0 : no blinking
    // 1 : both eyes blinking
    // 2 : left eye blinking
    // 3 : right eye blinking
    eyeBlink = function( leftX, leftY, rightX, rightY ){
	if( (leftX == 0. || leftY == 0.) || (rightX == 0. || rightY == 0.) ){
	    
	    if( ( leftX == 0. || leftY == 0. ) && ( rightX == 0. || rightY == 0. ) ){
		return 1;
	    }
	    else if( leftX == 0. || leftY == 0. ){
		return 2;
	    }
	    else{
		return 3;
	    }
	    
	}
	else
	    return 0;
	
    }
    
    //Initialize the connection via socket;io
    function initSocket(){
	console.log("init socket");
	socket = io("http://localhost:3000");
	
	//When data received from the server
	socket.on('coordinates', function( leftX, leftY, rightX, rightY ){

	    blinkValue = eyeBlink(leftX, leftY, rightX, rightY);
	    
	    //Filtering eyes blinking
	    if(blinkValue != 1){//no both eyes blinking (in this case, we do nothing)
		if(blinkValue != 0){// no blinking
		    
		    if(blinkValue == 2){// left eye
			leftX = rightX;
			leftY = rightY;		    
		    }
		    if(blinkValue == 3){// right eye
			rightX = leftX;
			rightY = leftY;		    
		    }
		}

	    	leftX = leftX*screen.width - offsetX;
	    	leftY = leftY*screen.height - offsetY;
		
	    	rightX = rightX*screen.width - offsetX;
	    	rightY = rightY*screen.height - offsetY;
	    	addNewCoordinates(leftX, leftY, rightX, rightY);		    
		
		
	    }
	    // if( (leftX != 0. && leftY != 0.) || (rightX != 0. && rightY != 0.) ){

	    // 	if( leftX == 0. || leftY == 0. ){
	    // 	    leftX = rightX;
	    // 	    leftY = rightY;
	    // 	}
		
	    // 	if( rightX == 0. || rightY == 0. ){
	    // 	    rightX = leftX;
	    // 	    rightY = leftY;
	    // 	}
		
	    // 	leftX = leftX*screen.width - offsetX;
	    // 	leftY = leftY*screen.height - offsetY;

	    // 	rightX = rightX*screen.width - offsetX;
	    // 	rightY = rightY*screen.height - offsetY;

	    // 	console.log(leftX, leftY, rightX, rightY);
		
	    // 	addNewCoordinates(leftX, leftY, rightX, rightY);
	    // }

	    update();
	    
	});
    }

    
    //Averaging the last "CURSOR_MEAN_BUFFER_SIZE" values received    
    function updateCursor(){
	
	var sum_x=0;
	var sum_x2=0;
	
	var sum_y=0;
	var sum_y2=0;
	
	var sum_xy=0;
	
	var count=0;
	
	for(var i=0; i<bufferX.length; ++i){
	    
	    sum_x += bufferX[i];
	    sum_x2 += bufferX[i] * bufferX[i];
	    sum_y += bufferY[i];
	    sum_y2 += bufferY[i] * bufferY[i];
	    count++;
	    
	}
	
	var Ex = sum_x/count;
	var Ex2 = sum_x2/count;
	var Ey = sum_y/count;
	var Ey2 = sum_y2/count;

	var COVxx = Ex2-Ex*Ex;
	var COVyy = Ey2-Ey*Ey;
	
	mean_x = Ex;
	mean_y = Ey;
	
	var1 = Math.sqrt(COVxx) ;
	var2 = Math.sqrt(COVyy) ;
	
	angle = 0.;
	
    }

    //Kepping only the last "CURSOR_MEAN_BUFFER_SIZE" values 
    function updateArrays(leftX, leftY, rightX, rightY){
	bufferX[X_it] = (leftX+rightX)/2;
	X_it = (X_it+1)%CURSOR_MEAN_BUFFER_SIZE;

	bufferY[Y_it] = (leftY+rightY)/2;
	Y_it = (Y_it+1)%CURSOR_MEAN_BUFFER_SIZE;
    }
    
    return {
	update : update,
	init : init,
	getPos : getPos,
	setFixationCallback: setFixationCallback
    };
    
}]);
