angular.module("textViewer").factory('ParagraphModel', ['WordModel', 'collisions', function(WordModel, collisions){ 
    var paragraphCount = 0;
    var PROCESS_THRESHOLD = 0.6;//proportion of marked word in a paragraph to consider it as read
    
    function ParagraphModel() {
	this.id = paragraphCount++;//unique identifier
	this.words = [];//words list
	this.element = null;//associated element to retrieve client rect
	this.read = false;//true if this word as been considered as read by the user
	this.selected = false;//true if this paragraph was selected during the last call to TextModel.updateReadMode()
	this.newSelected = false;//true if this paragraph is in the current selection of TextModel.updateReadMode()
    }

    ParagraphModel.prototype.addWord = function(w) {
	
	if(w instanceof WordModel)
	    this.words.push(w);
	
    }

    ParagraphModel.prototype.toString = function() {
	
	var str = "("+this.id+")";
	var i;
	
	for(i=0; i<this.words.length; ++i) 
	    str += this.words[i].toString();
    
	return str;
    }

    ParagraphModel.prototype.createFromString = function(paragraphStr) {
	
	this.words.length = 0;
	var this_obj = this;
	var words = paragraphStr.split(/\b/);//we split the string at each beginning and ending of a word
	var w_object = null;
	
	for(var i=0; i<words.length; ++i)
	{
	    if(w_object == null)
		w_object = new WordModel();

	    if(words[i].match(/\w/gi) != null)//if it is a word
	    {
		w_object.value = words[i];
	    }
	    else//if it is not a word
	    {
		w_object.after = words[i];
		this_obj.addWord(w_object);
		w_object = null;
	    }
	}

	if(w_object != null)
	    this_obj.addWord(w_object);
    }

    //Collision test
    ParagraphModel.prototype.isInRect = function(x,y) {
	
	if(this.element == null)
	    return false;
	
	return collisions.isInRect(this.element.getBoundingClientRect(),x,y);
    }

    ParagraphModel.prototype.ellipseIntersect = function(ellipse) {
	
	if(this.element == null)
	    return false;
	
	return collisions.isInRect(this.element.getBoundingClientRect(),ellipse.x,ellipse.y) || collisions.intersectionEllipseRect(ellipse, this.element.getBoundingClientRect());
    }    

    ParagraphModel.prototype.getWordAt = function(x,y) {
	
	var i
	for(i=0; i<this.words.length; ++i)
	    if(this.words[i].isInRect(x,y))
		return this.words[i];
	
	return null;
    }

    
    ParagraphModel.prototype.getWordsAtEllipse = function(ellipse) {
	var res = [];
	for(var i=0; i<this.words.length; ++i){
	    if(this.words[i].ellipseIntersect(ellipse))
		res.push(this.words[i]);
	}
	
	return res;
    }

    ParagraphModel.prototype.getMarkedWordsCount = function() {
	
	var i;
	var count = 0;
	
	for(i=0; i<this.words.length; ++i) 
	    if(this.words[i].marked == true)
		count++;
	
	return count;
    }

    ParagraphModel.prototype.getReadWordsCount = function() {
	
	var i;
	var count = 0;
	
	for(i=0; i<this.words.length; ++i) 
	    if(this.words[i].read == true)
		count++;
	
	return count;
    }

    ParagraphModel.prototype.updateWords = function(ellipse) {

	var w = this.getWordsAtEllipse(ellipse);
	for(var i=0; i<w.length; ++i)
	    w[i].marked = true;
    }

    ParagraphModel.prototype.process = function() {
	if(!this.read){
	    var n = this.getMarkedWordsCount();
	    var nt = this.words.length;
	    
	    if(n/nt  > PROCESS_THRESHOLD){
		this.read = true; 
		var i; 
		for(i=0; i<this.words.length; ++i){
		    this.words[i].read= true;
		    this.words[i].color = "rgb(0,0,255)";	
		}
		
	    }
	}
    }
    
    return ParagraphModel;
}]);
