angular.module("textViewer").factory('TextModel', ['ParagraphModel', 'collisions', function(ParagraphModel, collisions){

    
    function TextModel() {
    	this.id = null;
    	this.paragraphs = [];//paragraphs list
    	this.element = null;//associated element to retrieve client rect
    }

    TextModel.prototype.addParagraph = function(p) {
	
	if(p instanceof ParagraphModel) 
	    this.paragraphs.push(p);
	
    }

    TextModel.prototype.toString = function() {
	var str = "";
	
	for(i=0; i<this.paragraphs.length; i++)
	    str = str + this.paragraphs[i].toString()+"\n";
	
	return str;
    }

    //Creates a TextModel by parsing the text
    TextModel.prototype.createFromString = function(textStr, textId) {
	this.id = textId;
	this.paragraphs.length = 0;
	
	var this_obj = this;
	var paragraphs = textStr.split('\n');
	
	paragraphs.forEach(function(p,i){
	    
	    var p_object = new ParagraphModel();
	    
	    p_object.createFromString(p);
	    this_obj.addParagraph(p_object);
	    
	});
    }

    //Collision test
    TextModel.prototype.isInRect = function(x,y) {
	
	if(this.element == null)
	    return false;
	
	return collisions.isInRect(this.element.getBoundingClientRect(),x,y);
    }

    
    TextModel.prototype.ellipseIntersect = function(ellipse) {
	
	if(this.element == null)
	    return false;
	
	return collisions.isInRect(this.element.getBoundingClientRect(),ellipse.x,ellipse.y) || collisions.intersectionEllipseRect(ellipse, this.element.getBoundingClientRect());
    }


    TextModel.prototype.getTotalMarkedWordsCount = function(){
	var count = 0;
	var i;
	for(i=0; i<this.paragraphs.length; ++i)
	    count += this.paragraphs[i].getMarkedWordsCount();
	
	return count;
    }

    TextModel.prototype.getTotalReadWordsCount = function(){
	var count = 0;
	var i;
	for(i=0; i<this.paragraphs.length; ++i)
	    count += this.paragraphs[i].getReadWordsCount();
	
	return count;
    }
    
    
    TextModel.prototype.getParagraphAt = function(x,y) {
	
	for(i=0; i<this.paragraphs.length; ++i)
	    if(this.paragraphs[i].isInRect(x,y))
		return this.paragraphs[i];
	
	return null;
    }

    TextModel.prototype.getParagraphsAtEllipse = function(ellipse) {
	var res = [];
	for(i=0; i<this.paragraphs.length; ++i)
	    if(this.paragraphs[i].ellipseIntersect(ellipse))
		res.push(this.paragraphs[i]);
	
	return res;
    }
    
    TextModel.prototype.updateReadMode = function(ellipse) {
	if(this.ellipseIntersect(ellipse)){//if the ellipse is on the text
	    var p = this.getParagraphsAtEllipse(ellipse);
	    for(var i=0; i<p.length; ++i){
		p[i].newSelected = true;
		p[i].updateWords(ellipse);
	    }
	}

	var p = this.paragraphs;
	for(var i=0; i<p.length; ++i){
	    if(p[i].selected && !p[i].newSelected){//if we just left this paragraph
		p[i].process();
	    }
	    if(!p[i].selected && p[i].newSelected){//if we just entered in this paragraph
		p[i].selected = true;
	    }
	    p[i].newSelected = false;
	}
	
    }
    
    return TextModel;
}]);
