describe('paragraphModel', function(){

    var nbWords = 122;
    var sampleText = "Lorem ipsum dolor sit amet consectetur adipiscing elit Vestibulum commodo ante eu ante tempus eget blandit ante efficitur Vestibulum hendrerit tristique velit et tempor erat volutpat id Duis blandit urna et congue dignissim neque lacus semper ante sed ullamcorper augue leo at tortor Aenean elit eros tempus ut velit ac venenatis tincidunt purus Sed ut justo quis dolor malesuada consectetur nec ut massa Interdum et malesuada fames ac ante ipsum primis in faucibus Suspendisse blandit auctor massa at tristique Donec sit amet quam vitae leo tempor sagittis non quis mauris Suspendisse rutrum suscipit urna mattis efficitur orci vestibulum id Nunc et leo vel eros blandit placerat facilisis vitae magna Integer sodales tortor vitae velit bibendum fringilla Ut finibus rutrum quam a lacinia";

    var sampleTextWithPunctuation = "Lorem ipsum dolor sit amet consectetur adipiscing elit Vestibulum commodo ante eu ante tempus eget blandit ante efficitur Vestibulum hendrerit tristique velit et tempor erat, volutpat id Duis blandit urna et congue dignissim neque lacus semper ante sed ullamcorper augue leo at tortor Aenean elit eros tempus. ut velit ac venenatis, tincidunt purus Sed ut justo quis dolor malesuada consectetur, nec ut massa Interdum et malesuada ;fames ac ante, ipsum primis in, faucibus Suspendisse blandit auctor massa at tristique Donec sit amet quam vitae leo tempor sagittis non quis mauris Suspendisse rutrum suscipit urna mattis efficitur orci vestibulum id Nunc et leo vel eros, blandit placerat facilisis vitae magna Integer sodales tortor vitae velit bibendum fringilla Ut finibus rutrum quam a lacinia.";

    var nbWords2 = 123;
    var sampleTextBeginningByPunctuation = " ? Lorem ipsum dolor sit amet consectetur adipiscing elit Vestibulum commodo ante eu ante tempus eget blandit ante efficitur Vestibulum hendrerit tristique velit et tempor erat, volutpat id Duis blandit urna et congue dignissim neque lacus semper ante sed ullamcorper augue leo at tortor Aenean elit eros tempus. ut velit ac venenatis, tincidunt purus Sed ut justo quis dolor malesuada consectetur, nec ut massa Interdum et malesuada ;fames ac ante, ipsum primis in, faucibus Suspendisse blandit auctor massa at tristique Donec sit amet quam vitae leo tempor sagittis non quis mauris Suspendisse rutrum suscipit urna mattis efficitur orci vestibulum id Nunc et leo vel eros, blandit placerat facilisis vitae magna Integer sodales tortor vitae velit bibendum fringilla Ut finibus rutrum quam a lacinia.";
    
    // var $log;
    
    beforeEach(angular.mock.module('textViewer'));
    
    beforeEach(inject(function(_WordModel_) {
    	WordModel = _WordModel_;
    }));

    beforeEach(inject(function(_ParagraphModel_) {
	ParagraphModel = _ParagraphModel_;
    })); 

    // Inject the $log service
    // beforeEach(inject(function(_$log_){  
    // 	$log = _$log_;
    // }));

    // Log debug messages in Karma
    // afterEach(function(){  
    // 	console.log($log.debug.logs);
    // }); 

    it('should be defined', function(){
	expect(ParagraphModel).toBeDefined(); 
    });
    
    it('should exist', function(){
	expect(ParagraphModel.prototype.createFromString).toBeDefined();
    });

    describe('createFromString function', function(){
	
	it('should create the good number of words, equal to 122', function(){
	    var p = new ParagraphModel(); 
	    
	    p.createFromString(sampleText);

	    expect(nbWords).toEqual(p.words.length);
	    
	});

	it('should create the good number of words, equal to 122, even with punctuation', function(){
	    var p = new ParagraphModel(); 
	    
	    p.createFromString(sampleTextWithPunctuation);

	    expect(nbWords).toEqual(p.words.length);
	});

	it('should create the good number of words, equal to 123, with punctuation at the beginning', function(){
	    var p = new ParagraphModel(); 
	    
	    p.createFromString(sampleTextBeginningByPunctuation);
	    
	    expect(nbWords2).toEqual(p.words.length);
	});

    });

    describe('addWord function', function(){
	
	it('should add a word', function(){
	    var p = new ParagraphModel();
	    var w = new WordModel();
	    w.value = "test";
	    
	    expect(p.words.length).toEqual(0);
	    p.addWord(w);
	    expect(p.words.length).toEqual(1);
	    expect(p.words[0].value).toEqual("test");
	});

    });

    describe('getMarked/getRead WordsCount functions', function(){
	
	it('should get 5 marked words', function(){
	    var p = new ParagraphModel();

	    for(var i = 0 ; i < 10 ; i++){
		var w = new WordModel();
		w.value = "word_"+i;
		if(i%2==0)
		    w.marked = true;

		p.addWord(w);
	    }

	    expect(p.getMarkedWordsCount()).toEqual(5);	    
	});

	it('should get 5 read words', function(){
	    var p = new ParagraphModel();

	    for(var i = 0 ; i < 10 ; i++){
		var w = new WordModel();
		w.value = "word_"+i;
		if(i%2==0)
		    w.read = true;

		p.addWord(w);
	    }

	    expect(p.getReadWordsCount()).toEqual(5);	    
	});

    });

    describe('process function', function(){
	
	it('should not set the paragraph to read', function(){
	    var p = new ParagraphModel();

	    for(var i = 0 ; i < 10 ; i++){
		var w = new WordModel();
		w.value = "word_"+i;
		if(i%2==0)
		    w.marked = true;

		p.addWord(w);
	    }

	    p.process;
	    expect(p.read).toBe(false);
	    
	});

	it('should set the paragraph to read', function(){
	    var p = new ParagraphModel();

	    for(var i = 0 ; i < 10 ; i++){
		var w = new WordModel();
		w.value = "word_"+i;
		w.marked = true;

		p.addWord(w);
	    }
	    
	    p.process();
	    expect(p.read).toBe(true);
	    
	});

    });
    
});
