angular.module("textViewer").controller('textViewerController', ['$scope', 'eyeTrackerFactory', '$interval', 'TextModel', '$http', function($scope, eyeTrackerFactory, $interval, TextModel, $http){

    var RESEND_FREQUENCY = 10000;//Every 10000 ms, send an update request to the eye tracker module, in case the connection is lost
    var REFRESH_DISPLAY_RATE = 10;//duration in ms between two refresh of the display
    
    $scope.text = new TextModel();

    $scope.fixations = [];//list of fixations
    $scope.collisions = [];//list of collision ellipses

    //updateDisplay 
    $scope.updateDisplay = function(){
	
	$scope.pos = eyeTrackerFactory.getPos();
	$scope.displayPos = eyeTrackerFactory.getPos();
	$scope.displayPos.x += window.pageXOffset;
	$scope.displayPos.y += window.pageYOffset;

	deleteDisplayRects();
	
	if($scope.showRect)
	    $scope.displayRects();
    }
    
    eyeTrackerFactory.init();//Initialize the eye tracker module
    
    eyeTrackerFactory.setFixationCallback(function(fixation){
	
	var collisionEllipse = {
	    x: fixation.x,
	    y: fixation.y,
	    var_x: fixation.var_x,
	    var_y: $scope.lineSpace+$scope.fontSize*2
	};
	
	if($scope.text.isInRect(fixation.x, fixation.y) || $scope.text.ellipseIntersect(collisionEllipse)){
	    
	    var displayFixation = {
		x: fixation.x+window.pageXOffset,
		y: fixation.y+window.pageYOffset,
		var_x: fixation.var_x,
		var_y: fixation.var_y
	    };
	    
	    $scope.fixations.push(displayFixation);

	    if($scope.readMode) {
		$scope.text.updateReadMode(collisionEllipse);
		collisionEllipse.x +=window.pageXOffset;
		collisionEllipse.y +=window.pageYOffset;
		$scope.collisions.push(collisionEllipse);	
	    }
	    
	}
	
    });
    
    $scope.updateDisplay();
    
    //Update the eye tracker module
    $scope.updatePos = function(){
	eyeTrackerFactory.update();
    }

    $scope.updatePos();
    $interval($scope.updatePos, RESEND_FREQUENCY);
    $interval($scope.updateDisplay, REFRESH_DISPLAY_RATE);

    //Inputs of the parameters interface
    $scope.fontSize = 25;//text font size
    $scope.lineSpace =14;//text line spacing

    //Buttons of the parameters interface
    $scope.showRect = false;//if true, displays bounding rectangles of each text/paragraph/word
    $scope.showReticle = false;//if true, displays the reticle's ellipse returned by the eye tracker
    $scope.showParams = false;//if true, show hidden parameter's menu
    $scope.readMode = true;//if true, read word detection is activated
    $scope.colorWords = false;//if true, displays each marked word in red
    $scope.doneReading = false;//if true, current text reading session is ended
    $scope.showFixations = false;//if true, displays each fixations on the screen
    $scope.showCollisions = false;//if true, displays each collisions on the screen

    //Returns the number of words read over the course of the session
    $scope.nbWordsRead = function(){
	return $scope.text.getTotalReadWordsCount();
    }

    $scope.reticleColor = "blue";
    $scope.fixationColor = "red";
    $scope.collisionColor = "cyan";

    $scope.sendSessionData = function(){
	console.log("sendSession");

	var wordsArray = [];
	var stringWordsArray = "";
	
	for(var i = 0 ; i < $scope.text.paragraphs.length ; i++){
	    if($scope.text.paragraphs[i].read == true){
		
		for(var j = 0 ; j < $scope.text.paragraphs[i].words.length ; j++){
		    
		    wordsArray.push($scope.text.paragraphs[i].words[j].value);
		    
		    if(j<$scope.text.paragraphs[i].words.length-1)
			stringWordsArray+=$scope.text.paragraphs[i].words[j].value+";";
		    else
			stringWordsArray+=$scope.text.paragraphs[i].words[j].value;
		    
		}
		
	    }
	}

	var data = {"userId" : $scope.user_id, "textId" : $scope.text_id, "nbWordsRead" : $scope.nbWordsRead(), "wordsArray" : wordsArray}; 

	var jsonData = JSON.stringify(data);
	
	$http.post('sendSessionData.php', jsonData);//.success(function (data) {
	//});
	
    }
    

    /* Parses raw text and create the model */
    $scope.createModelFromText = function() {
	
	var tmp_elem = $scope.text.element;
	$scope.text = new TextModel();
	$scope.text.element = tmp_elem;
	$scope.text.createFromString($scope.text_brut, $scope.text_id);
	
    }

    $scope.init = function() {
	$scope.createModelFromText();
    }

    //Display the bounding rects of the words (debug)
    function addDisplayBoundingRect(r) {
	if(r == null)
	    console.log("null rect");
	else {
	    
	    x_pos = r.left+window.pageXOffset;
	    y_pos = r.top+window.pageYOffset;

	    div = document.createElement("div");
	    div.className = "boundingRectDisplay";
	    div.setAttribute("style","position:absolute;left:"+x_pos+"px;top:"+y_pos+"px;width:"+r.width+"px;height:"+r.height+"px;border:1px solid #000;pointer-events:none;")
	    document.body.appendChild(div);
	    
	}
    }
    
    //Debug : display bounding rects of all words
    $scope.displayRects = function (){
	
	//display text bounding rect
	addDisplayBoundingRect($scope.text.element.getBoundingClientRect());
	
	for(i=0; i<$scope.text.paragraphs.length; ++i){
	    addDisplayBoundingRect($scope.text.paragraphs[i].element.getBoundingClientRect()); 
	    for(j=0; j<$scope.text.paragraphs[i].words.length; ++j)
		addDisplayBoundingRect($scope.text.paragraphs[i].words[j].element.getBoundingClientRect());	

	}
	
    }

    //Debug : delete all bounding rects displayed by $scope.displayRects()
    function deleteDisplayRects(){
	
	brects = document.getElementsByClassName("boundingRectDisplay");
	
	r = [];
	
	for(i=0; i< brects.length; i++)
	    r[i] = brects[i];

	for(i=0; i<r.length; i++)
	    r[i].parentNode.removeChild(r[i]);
	
    }
    
}]);
