<?php

//------This file handles all the databases accesses------
include "libs/PorterStemmer.php";

//Pdo instance of the database
$GLOBALS['db'] = new PDO(
    'mysql:host=localhost;dbname=PdpEyeTracker',//Host and database name
    'root',//Username
    ''//Password
);

function getAllTextsIds(){
    $str = "SELECT id FROM texte";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute();
    $result = $req->fetchAll();
    return $result;
}

function getTextByType($type){
    $req = $GLOBALS['db']->prepare("SELECT * FROM texte where type=:t ");
    $req->execute(array(':t' => $type));
    $result = $req->fetchAll();
    return $result;
}

function getTextById($id){
    
    $req = $GLOBALS['db']->prepare("SELECT * FROM texte where id=:id");
    $req->execute(array(':id' => $id));
    
    $result = $req->fetchAll();
    return $result;    
}

//----------User managment-------------

function insertNewUser($name, $mail, $password, $salt){
    
    $str = "INSERT INTO  `PdpEyeTracker`.`Users` (`name`, `mail`, `password`, `salt`) VALUES (:name, :mail, :password, :salt);";
    
    $req = $GLOBALS['db']->prepare($str);
    $req->execute( array( 'name' => $name, 'mail' => $mail, 'password' => $password, 'salt' => $salt));
}

//Checks if the user name exists, and if the hashed password matches the one in the database associated with the user
function getUser($name, $hashedPW){
    $str = "SELECT id, name, mail FROM Users where name = :name and password = :pw ";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute( array( 'name' => $name, 'pw' => $hashedPW));
    $result = $req->fetchAll();
    return $result;
}

//Checks if the supplied user exists
function userExists($name){
    $str = "SELECT id FROM Users where name = :name ";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute( array( 'name' => $name));
    $result = $req->fetchAll();
    
    if($result == null)
	return false;
    else
	return true;    
}

function getSalt($name){
    $str = "SELECT salt FROM Users where name = :name ";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute( array( 'name' => $name));
    $result = $req->fetchAll();
    return $result[0][0];
}

//---------------/User Managment/---------------


//Creates a session, adding all the words read during the session in the 'wordsReadInSession' Table
function insertSession($userId, $textId, $nbWordsRead, $wordsArray){
    
    $str = "INSERT INTO  `PdpEyeTracker`.`Session` (`userId`, `textId`, `nbWordsRead`, `date`) VALUES (:userId, :textId, :nbWordsRead, NOW());"; 
    $req = $GLOBALS['db']->prepare($str);
    $req->execute( array( 'userId' => $userId, 'textId' => $textId, 'nbWordsRead' => $nbWordsRead)); 

    $str = "SELECT id FROM Session ORDER BY id DESC LIMIT 1";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute();
    $result = $req->fetchAll();
    $idSession = $result[0][0];

    foreach($wordsArray as $w){
	$stemmedWord = PorterStemmer::Stem($w);
	$str = "SELECT id FROM words WHERE word=:wordValue";
	$req = $GLOBALS['db']->prepare($str);	
	$req->execute(array('wordValue' => $stemmedWord));
	$result = $req->fetchAll();
	$idWord = $result[0][0];
	
	$str = "INSERT INTO  `PdpEyeTracker`.`wordReadInSession` (`idSession`, `idWord`, `idUser`) VALUES (:idSession, :idWord, :idUser);"; 
	$req = $GLOBALS['db']->prepare($str);
	$req->execute( array( 'idSession' => $idSession, 'idWord' => $idWord, 'idUser'=>$userId));	
    }
    
    
}

//debug
function tempFillAll(){
    $str = "SELECT id FROM texte";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute();
    $result = $req->fetchAll();

    foreach($result as $r){
	fillWordsInText($r[0]);
    }
}

//Creates the entries in the association table wordsInTexts
//Every time a text is added, this function must be executed
function fillWordsInText($textId){
    $words = getAllWordsText($textId);
    foreach($words as $w){
	$str = "INSERT INTO  `PdpEyeTracker`.`wordsInTexts` (`idText`, `idWord`) VALUES (:idText, :idWord);"; 
	$req = $GLOBALS['db']->prepare($str);
	$req->execute( array( 'idText' => $textId, 'idWord' => $w));	    
    }    
}

function getAllWordsUser($userId){

    $str = "SELECT idWord FROM wordReadInSession WHERE idUser=:userId GROUP BY idWord";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('userId'=>$userId));
    $result = $req->fetchAll();
    
    return $result;
}

//Debug
function getAllWordsText2($textId){
    
    $str = "SELECT contenu FROM texte WHERE id=:textId";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('textId'=>$textId));
    $result = $req->fetchAll();

    $texte = $result[0][0];

    $s = preg_replace('/[^a-z0-9]+/i', ' ', $texte);//Remove punctuation

    $words = explode(' ', $s);//Getting array of words

    $wordsIds = array();
    foreach($words as $w){//Getting ids of each words

	$stemmedWord = PorterStemmer::Stem($w);
	
	$str = "SELECT id FROM words WHERE word=:w";
	$req = $GLOBALS['db']->prepare($str);
	$req->execute(array('w'=>$stemmedWord));
	$result = $req->fetchAll();
	if(!empty($result[0][0]))
	    array_push($wordsIds, $result[0][0]);
    }

    $wordsIdsNoDuplicates = array_unique($wordsIds, SORT_REGULAR);//Removing duplicates
    
    return $wordsIdsNoDuplicates;
}

function getAllWordsText($textId){
    
    $str = "SELECT idWord FROM wordsInTexts WHERE idText = :textId";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('textId'=>$textId));
    $result = $req->fetchAll();
    $res = array();
    
    foreach($result as $r){
	array_push($res, $r[0]);
    } 
    return $res;
}

function getWordFromId($id){
    $str = "SELECT word FROM words WHERE id = :id";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('id'=>$id));
    $result = $req->fetchAll();
    return $result[0][0];
    
}

function getNbWordsReadForUser($id){
    
    $str = "select SUM(nbWordsRead) from Session where userId=:id";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('id'=>$id));
    $result = $req->fetchAll();
    
    return $result[0][0];
}

function getSessionsForUser($id){
    
    $str = "SELECT * FROM Session WHERE userId=:id ORDER BY date";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('id'=>$id));
    $result = $req->fetchAll();

    return $result;
}

function getNbUniqueWordsReadByUser($id){
    
    $str = "select count(*) from (SELECT count(*) FROM wordReadInSession WHERE idUser=:id group by idWord) as r1";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute(array('id'=>$id));
    $result = $req->fetchAll();

    return $result[0][0];    
}

function getNbUniqueWords(){
    $str = "select count(*) from words";
    $req = $GLOBALS['db']->prepare($str);
    $req->execute();
    $result = $req->fetchAll();

    return $result[0][0];    
}
?>
